@@ -1,16 +1,4 @@
 package App::Tel;
-use strict;
-use warnings;
-use Expect qw( exp_continue );
-use POSIX qw(:sys_wait_h :unistd_h); # For WNOHANG
-use Hash::Merge::Simple qw (merge);
-use Module::Load;
-use App::Tel::HostRange qw (check_hostrange);
-use App::Tel::Passwd;
-use App::Tel::Color;
-use Time::HiRes qw ( sleep );
-use v5.10;
-
 
 =head1 NAME
 
@@ -18,12 +6,7 @@ App::Tel - A script for logging into devices
 
 =head1 VERSION
 
-0.2015.07
-
-=cut
-
-our $VERSION = '0.2015.07';
-
+0.2005
 
 =head1 SYNOPSIS
 
@@ -45,27 +28,20 @@ under the same terms as Perl itself.
 
 =cut
 
-#### GLOBALS
-# For reasons related to state I needed to make $_winch_it global
+use strict;
+use warnings;
+use Expect qw( exp_continue );
+use IO::Stty;
+use POSIX qw(:sys_wait_h :unistd_h); # For WNOHANG and isatty
+use Hash::Merge::Simple qw (merge);
+use Module::Load;
+use v5.10;
+
+our $VERSION = eval '0.2005';
+
+# For reasons related to state I needed to make $winch_it global
 # because it needs to be written to inside signals.
-my $_winch_it=0;
-
-sub _winch_handler {
-    $_winch_it=1;
-}
-
-sub _winch {
-    my $session = shift->{'session'};
-    # these need to be wrapped in eval or you get Given filehandle is not a
-    # tty in clone_winsize_from if you call winch() under a scripted
-    # environment like rancid (or just under par, or anywhere there is no pty)
-    eval {
-        $session->slave->clone_winsize_from(\*STDIN);
-        kill WINCH => $session->pid if $session->pid;
-    };
-    $_winch_it=0;
-    $SIG{WINCH} = \&_winch_handler;
-}
+my $winch_it=0;
 
 =head1 METHODS
 
@@ -78,16 +54,15 @@ Creates a new App::Tel object.
 =cut
 
 sub new {
+
     my $infile = IO::File->new;
     $infile->IO::File::fdopen( \*STDIN, 'r' );
 
     my $self = {
-        'stdin'         => Expect->exp_init($infile),
-        'stdin_fileno'  => $infile->fileno,
-        'connected'     => 0,
-        'enabled'       => 0,
-        'title_stack'   => 0,
-        'log_stdout'    => 1,
+        'stdin' => Expect->exp_init($infile),
+        'connected' => 0,
+        'enabled' => 0,
+        'title_stack' => 0,
     };
 
     bless($self, 'App::Tel');
@@ -110,7 +85,6 @@ sub disconnect {
     my $self = shift;
     my $hard = shift;
     $self->{profile} = {};
-    $self->{timeout} = $self->{opts}->{t} ? $self->{opts}->{t} : 90;
     $self->{banners} = undef;
     $self->{methods} = ();
     $self->connected(0);
@@ -127,28 +101,10 @@ sub disconnect {
     }
 }
 
-
-=head2 send
-
-    $self->send("text\r");
-
-Wrapper for Expect's send() method.
-
-=cut
-
 sub send {
     return shift->{'session'}->send(@_);
 }
 
-=head2 expect
-
-    $self->expect("text");
-
-Wrapper for Expect's expect() method.  If you don't specify a timeout this
-will use the default script timeout.
-
-=cut
-
 sub expect {
     my $self = shift;
     if ($#_ < 2) {
@@ -158,6 +114,32 @@ sub expect {
     }
 }
 
+=head2 load_syntax
+
+   $self->load_syntax('Cisco');
+
+This attempts to load syntax highlighting modules.  In the above example,
+Cisco would append Colors to the end and get CiscoColors.pm.  If it can't find
+the module it just won't load it.
+
+Multiple files can be chain loaded by using plus:
+
+    $self->load_syntax('Default+Cisco');
+
+=cut
+
+sub load_syntax {
+    my $self = shift;
+    for(split(/\+/, shift)) {
+        my $module = 'App::Tel::'.$_.'Colors';
+
+        eval {
+            Module::Load::load $module;
+            push(@{$self->{colors}}, $module->new);
+        };
+    }
+}
+
 =head2 load_config
 
 Loads the config from /etc/telrc, /usr/local/etc/telrc, $ENV{HOME}/.telrc2, or
@@ -188,9 +170,9 @@ sub load_config {
         warn "No configuration files loaded. You may need to run mktelrc.";
     }
 
-    # load global syntax highlighting things if found
+    # load global syntax things
     for(@{$config->{syntax}}) {
-        App::Tel::Color::load_syntax($_,$self->{opts}->{d});
+        $self->load_syntax($_);
     }
     return $config;
 }
@@ -304,15 +286,8 @@ sub _banners {
     return $self->{banners};
 }
 
-=head2 rtr_find
-
-    my $profile = $self->rtr_find($regex);
-
-Find the router by hostname/regex and load the config associated with it.
-Load a profile for it if there is one.
-
-=cut
-
+# find the router by hostname/regex and load the config associated with it.
+# Load a profile for it if there is one.
 sub rtr_find {
     my $self = shift;
     my $host = shift;
@@ -321,7 +296,7 @@ sub rtr_find {
 
     foreach my $h (@{$config->{rtr}}) {
         my $h2 = $h->{regex};
-        if ($host =~ /$h2/i || check_hostrange($h2, $host)) {
+        if ($host =~ /$h2/i) {
             $profile=merge($profile, $h);
             last;
         }
@@ -375,7 +350,7 @@ sub profile {
         }
         # load syntax highlight
         if ($profile->{syntax}) {
-            App::Tel::Color::load_syntax($profile->{syntax},$self->{opts}->{d});
+            $self->load_syntax($profile->{syntax});
         }
         $profile->{profile_name}=$_;
     }
@@ -386,30 +361,68 @@ sub profile {
     return $profile;
 }
 
-sub _stty_rows {
-    my $new_rows = shift;
-    eval {
-        Module::Load::load Term::ReadKey;
-        my ($columns, $rows, $xpix, $ypix) = GetTerminalSize(\*STDOUT);
-        SetTerminalSize($columns, $new_rows, $xpix, $ypix, \*STDOUT);
-    };
 
-    warn $@ if ($@);
+sub _input_password {
+    my $prompt = shift;
+    $prompt ||= '';
+    my $old_mode=IO::Stty::stty(\*STDIN,'-g');
+    print "Enter password for $prompt: ";
+    IO::Stty::stty(\*STDIN,'-echo');
+    chomp(my $password=<STDIN>);
+    IO::Stty::stty(\*STDIN,$old_mode);
+    return $password;
 }
 
-=head2 password
+# just examples for keyring and keepass
+sub _keyring {
+    my $user = shift;
+    my $domain = shift;
+    my $group = shift;
 
-    my $password = $self->password;
+    # TODO: needs to warn on some kind of failure?
+    eval {
+        load Passwd::Keyring::Auto, 'get_keyring';
+        my $keyring = get_keyring(app=>"tel script", group=>$group);
+        my $pass = $keyring->get_password($user, $domain);
+        if (!$pass) {
+            $pass = _input_password($domain);
+            $keyring->set_password($user, $pass, $domain);
+        }
+        return $pass;
+    };
+}
 
-This pulls the password from the config.  If the password is blank it checks
-to see if you have a password manager, then tries to load the password from
-there.  It then tries to use an OS keyring.
+# this works but needs error checking and documentation.  You also can't store
+# your keepass password in a keyring yet, but you can store it in an
+# environment variable and set $keepass_passwd to be $ENV{KEEPASSPWD}
+sub _keepass {
+    my $keepass_file = shift;
+    my $keepass_passwd = shift;
+    my $keepass_entry = shift;
 
-By default it will pull the regular password.  To pull the enable password
-you can call it with $self->password('enable');
+    if ($keepass_passwd eq 'KEYRING') {
+        $keepass_passwd = _keyring('KEEPASS','KEEPASS','KEEPASS');
+    }
 
-=cut
+    # TODO: needs to warn on unlock failure
+    eval {
+        load File::KeePass;
 
+        my $k = File::KeePass->new;
+        $k->load_db($keepass_file, $keepass_passwd);
+
+        $k->unlock;
+
+        my $e = $k->find_entry({title => $keepass_entry});
+        return $e->{'password'};
+    };
+}
+
+# This pulls the password from the config.  If the password is blank it checks
+# to see if you have Keyring support, or Keepass support, then tries to load
+# the password from there.
+# By default it will pull the regular password.  To pull the enable password
+# you can call it with $self->password('enable');
 sub password {
     my $self = shift;
     my $type = shift;
@@ -418,6 +431,7 @@ sub password {
 
     $type ||= 'password';
 
+    # could use: not $type ~~ [ 'password', 'enable' ] but smartmatch is thinged
     warn "Unknown password type $type" if ($type ne 'password' && $type ne 'enable');
 
     if (defined($profile->{$type}) && $profile->{$type} ne '') {
@@ -431,9 +445,20 @@ sub password {
     }
 
     # if we get here, the password is blank so try other means
-    my $pass = App::Tel::Passwd::load_from_profile($profile);
-    if ($pass ne '') {
-        return $pass;
+    # what do we want to do if they want to store their router password in
+    # keepass and their keepass password in keyring?  We should maybe use
+    # environment variables to control where we load from and what order, but
+    # I'm not sure
+
+    if (defined($profile->{keepass_file})) {
+        if (my $pass = _keepass($profile->{keepass_file},
+                                $profile->{keepass_passwd},
+                                $profile->{keepass_entry}))
+        {
+            if ($pass ne '') {
+                return $pass;
+            }
+        }
     }
 
     # I was wondering how to decide what to prompt for, but I think it should
@@ -441,25 +466,19 @@ sub password {
     # for profile name and then save as profile name. If they want to be
     # explicit they should specify the password format as KEYRING or
     # something.. I dunno.
-    App::Tel::Passwd::keyring($profile->{user}, $profile->{profile_name}, $profile->{profile_name});
+    _keyring($profile->{user}, $profile->{profile_name}, $profile->{profile_name});
 
     # if they make it here and still don't have a password then none was
     # defined anywhere and we probably should prompt for one.  Consider
     # turning off echo then normal read.
-    return App::Tel::Passwd::input_password($router);
+    return _input_password($router);
+    return 'undef';
 }
 
-=head2 session
-
-    my $session = $self->session;
-
-Expect won't let you reuse a connection that has spawned a process, so you
-can call this with an argument to reset the session.  If called without an
-argument it will return the current session (If it exists) or create a new
-session.
-
-=cut
-
+# Expect won't let you reuse a connection that has spawned a process, so you
+# can call this with an argument to reset the session.  If called without an
+# argument it will return the current session (If it exists) or create a new
+# session.
 sub session {
     my $self = shift;
     my $renew = shift;
@@ -471,22 +490,27 @@ sub session {
     $session = new Expect;
 
     # install sig handler for window size change
-    $SIG{WINCH} = \&_winch_handler;
+    $SIG{WINCH} = \&winch_handler;
 
+    sub winch_handler {
+        $winch_it=1;
+    }
 
     $session->log_stdout(1);
     $self->{'session'} = $session;
     return $session;
 }
 
-=head2 connect
-
-    my $session = $self->connect('routername');
-
-This sets up the session.  If there already is a session open it closes and opens a new one.
-
-=cut
+sub winch {
+    my $session = shift->{'session'};
+    $session->slave->clone_winsize_from(\*STDIN);
+    kill WINCH => $session->pid if $session->pid;
+    $winch_it=0;
+    $SIG{WINCH} = \&winch_handler;
+}
 
+# this sets up the session.  If there already is a session open it closes and
+# opens a new one.
 sub connect {
     my $self = shift;
     my @arguments = shift;
@@ -497,20 +521,9 @@ sub connect {
     return $session;
 }
 
-=head2 connected
-
-    if ($self->connected);
-or
-    $self->connected(1);
-
-Returns connection status, or sets the status to whatever value is supplied to
-the method.
-
-Note: This isn't the session state, but an indicator that our session has
-gotten through the login stage and is now waiting for input.  i.e., the router
-is at a prompt of some kind.
-
-=cut
+# connected, returns connection status.  Sets status if given input.
+# this isn't the session state, but an indicator that our session has
+# gotten through the login stage and is now waiting for input.
 
 sub connected {
     my $self = shift;
@@ -522,13 +535,7 @@ sub connected {
     return $self->{connected};
 }
 
-=head2 enable
-
-    my $enabled = $self->enable;
-
-if enablecmd is set then this method attempts to enable the user
-
-=cut
+# if enablecmd is set then this method attempts to enable the user
 
 sub enable {
     my $self = shift;
@@ -552,16 +559,6 @@ sub enable {
     return $self->{enabled};
 }
 
-=head2 login
-
-    my $something = $self->login("hostname");
-
-Cycles through the connection methods trying each in the order specified by
-the profile until we successfully connect to the host.  Returns connected
-status (true or false).
-
-=cut
-
 sub login {
     my $self = shift;
     my $hostname = shift;
@@ -610,14 +607,12 @@ sub login {
         elsif ($_ eq 'exec')    { $self->connect($hostname); }
         else { die "No program defined for method $_\n"; }
 
-        # suppress stdout if needed
-        $self->{'session'}->log_stdout($self->{log_stdout});
-
         # need to make this optional
         # also need to make it display whatever the user cares about.
         print "\e[22t\033]0;$_ $hostname\007";
         $self->{title_stack}++;
         $SIG{INT} = sub { for (1..$self->{title_stack}) { print "\e[23t"; } $self->{title_stack}=0; };
+
         $self->expect($self->{timeout},
                 @{$self->_banners},
                 @dynamic,
@@ -687,16 +682,6 @@ sub logging {
     $self->session->log_file("/tmp/$file.log");
 }
 
-=head2 interact
-
-    $self->interact($input, $escape);
-
-This is a copy of Expect's interact() command.  It's been rewritten in parts
-to customize it for our needs, but might be very similar otherwise.  It's
-mainly a setup script for the call to interconnect().
-
-=cut
-
 sub interact {
     my $self = shift;
     my $session = $self->{'session'};
@@ -738,112 +723,102 @@ sub interact {
     $session->manual_stty($old_manual_stty_val);
 }
 
-=head2 interconnect
-
-    $self->interconnect(@handles);
-
-This is a copy of Expect's interconnect() method that has been modified to
-support the new things we needed.  The main differences are colorize and our
-winch handler.
-
-Future versions might support AnyEvent::IO or AnyEvent::Socket, but I might
-contribute that back to Expect's core stuff.  I might also try to figure out
-how to make the colorize stuff more hookable so we could use Expect's methods
-without rewriting them.
-
-=cut
-
 sub interconnect {
     my $self = shift;
-    my @handles = ($self->{'session'}, $_[0]);
+	my @handles = ($self->{'session'}, $_[0]);
 
-    my ( $nread );
-    my ( $rout, $emask, $eout );
-    my ( $escape_character_buffer );
-    my ( $read_mask, $temp_mask ) = ( '', '' );
+	#  my ($handle)=(shift); call as Expect::interconnect($spawn1,$spawn2,...)
+	my ( $nread );
+	my ( $rout, $emask, $eout );
+	my ( $escape_character_buffer );
+	my ( $read_mask, $temp_mask ) = ( '', '' );
 
-    # Get read/write handles
-    foreach my $handle (@handles) {
-        $temp_mask = '';
-        vec( $temp_mask, $handle->fileno(), 1 ) = 1;
-        $read_mask = $read_mask | $temp_mask;
-    }
-    if ($Expect::Debug) {
-        print STDERR "Read handles:\r\n";
-        foreach my $handle (@handles) {
-            print STDERR "\tRead handle: ";
-            print STDERR "'${*$handle}{exp_Pty_Handle}'\r\n";
-            print STDERR "\t\tListen Handles:";
-            foreach my $write_handle ( @{ ${*$handle}{exp_Listen_Group} } ) {
-                print STDERR " '${*$write_handle}{exp_Pty_Handle}'";
-            }
-            print STDERR ".\r\n";
-        }
-    }
+	# Get read/write handles
+	foreach my $handle (@handles) {
+		$temp_mask = '';
+		vec( $temp_mask, $handle->fileno(), 1 ) = 1;
+		$read_mask = $read_mask | $temp_mask;
+	}
+	if ($Expect::Debug) {
+		print STDERR "Read handles:\r\n";
+		foreach my $handle (@handles) {
+			print STDERR "\tRead handle: ";
+			print STDERR "'${*$handle}{exp_Pty_Handle}'\r\n";
+			print STDERR "\t\tListen Handles:";
+			foreach my $write_handle ( @{ ${*$handle}{exp_Listen_Group} } ) {
+				print STDERR " '${*$write_handle}{exp_Pty_Handle}'";
+			}
+			print STDERR ".\r\n";
+		}
+	}
 
-    #  I think if we don't set raw/-echo here we may have trouble. We don't
-    # want a bunch of echoing crap making all the handles jabber at each other.
-    foreach my $handle (@handles) {
-        unless ( ${*$handle}{"exp_Manual_Stty"} ) {
+	#  I think if we don't set raw/-echo here we may have trouble. We don't
+	# want a bunch of echoing crap making all the handles jabber at each other.
+	foreach my $handle (@handles) {
+		unless ( ${*$handle}{"exp_Manual_Stty"} ) {
 
-            # This is probably O/S specific.
-            ${*$handle}{exp_Stored_Stty} = $handle->exp_stty('-g');
-            print STDERR "Setting tty for ${*$handle}{exp_Pty_Handle} to 'raw -echo'.\r\n"
-                if ${*$handle}{"exp_Debug"};
-            $handle->exp_stty("raw -echo");
-        }
-        foreach my $write_handle ( @{ ${*$handle}{exp_Listen_Group} } ) {
-            unless ( ${*$write_handle}{"exp_Manual_Stty"} ) {
-                ${*$write_handle}{exp_Stored_Stty} =
-                    $write_handle->exp_stty('-g');
-                print STDERR "Setting ${*$write_handle}{exp_Pty_Handle} to 'raw -echo'.\r\n"
-                    if ${*$handle}{"exp_Debug"};
-                $write_handle->exp_stty("raw -echo");
-            }
-        }
-    }
+			# This is probably O/S specific.
+			${*$handle}{exp_Stored_Stty} = $handle->exp_stty('-g');
+			print STDERR "Setting tty for ${*$handle}{exp_Pty_Handle} to 'raw -echo'.\r\n"
+				if ${*$handle}{"exp_Debug"};
+			$handle->exp_stty("raw -echo");
+		}
+		foreach my $write_handle ( @{ ${*$handle}{exp_Listen_Group} } ) {
+			unless ( ${*$write_handle}{"exp_Manual_Stty"} ) {
+				${*$write_handle}{exp_Stored_Stty} =
+					$write_handle->exp_stty('-g');
+				print STDERR "Setting ${*$write_handle}{exp_Pty_Handle} to 'raw -echo'.\r\n"
+					if ${*$handle}{"exp_Debug"};
+				$write_handle->exp_stty("raw -echo");
+			}
+		}
+	}
 
-    print STDERR "Attempting interconnection\r\n" if $Expect::Debug;
+	print STDERR "Attempting interconnection\r\n" if $Expect::Debug;
 
-    # Wait until the process dies or we get EOF
-    # In the case of !${*$handle}{exp_Pid} it means
-    # the handle was exp_inited instead of spawned.
-    CONNECT_LOOP:
-    while (1) {
+	# Wait until the process dies or we get EOF
+	# In the case of !${*$handle}{exp_Pid} it means
+	# the handle was exp_inited instead of spawned.
+	CONNECT_LOOP:
 
-        # test each handle to see if it's still alive.
-        foreach my $read_handle (@handles) {
-            waitpid( ${*$read_handle}{exp_Pid}, WNOHANG )
-                if ( exists( ${*$read_handle}{exp_Pid} )
-                and ${*$read_handle}{exp_Pid} );
-            if (    exists( ${*$read_handle}{exp_Pid} )
-                and ( ${*$read_handle}{exp_Pid} )
-                and ( !kill( 0, ${*$read_handle}{exp_Pid} ) ) )
-            {
-                print STDERR
-                    "Got EOF (${*$read_handle}{exp_Pty_Handle} died) reading ${*$read_handle}{exp_Pty_Handle}\r\n"
-                    if ${*$read_handle}{"exp_Debug"};
-                last CONNECT_LOOP
-                    unless defined( ${ ${*$read_handle}{exp_Function} }{"EOF"} );
-                last CONNECT_LOOP
-                    unless &{ ${ ${*$read_handle}{exp_Function} }{"EOF"} }
-                    ( @{ ${ ${*$read_handle}{exp_Parameters} }{"EOF"} } );
-            }
-        }
+	# Go until we have a reason to stop
+	while (1) {
 
-        my $nfound = select( $rout = $read_mask, undef, $eout = $emask, undef );
+		# test each handle to see if it's still alive.
+		foreach my $read_handle (@handles) {
+			waitpid( ${*$read_handle}{exp_Pid}, WNOHANG )
+				if ( exists( ${*$read_handle}{exp_Pid} )
+				and ${*$read_handle}{exp_Pid} );
+			if (    exists( ${*$read_handle}{exp_Pid} )
+				and ( ${*$read_handle}{exp_Pid} )
+				and ( !kill( 0, ${*$read_handle}{exp_Pid} ) ) )
+			{
+				print STDERR
+					"Got EOF (${*$read_handle}{exp_Pty_Handle} died) reading ${*$read_handle}{exp_Pty_Handle}\r\n"
+					if ${*$read_handle}{"exp_Debug"};
+				last CONNECT_LOOP
+					unless defined( ${ ${*$read_handle}{exp_Function} }{"EOF"} );
+				last CONNECT_LOOP
+					unless &{ ${ ${*$read_handle}{exp_Function} }{"EOF"} }
+					( @{ ${ ${*$read_handle}{exp_Parameters} }{"EOF"} } );
+			}
+		}
+
+		# Every second? No, go until we get something from someone.
+		my $nfound = select( $rout = $read_mask, undef, $eout = $emask, undef );
 
         # Is there anything to share?  May be -1 if interrupted by a signal...
-        $self->_winch() if $_winch_it;
-        next CONNECT_LOOP if not defined $nfound or $nfound < 1;
+        $self->winch() if $winch_it;
+		next CONNECT_LOOP if not defined $nfound or $nfound < 1;
 
-        # Which handles have stuff?
-        my @bits = split( //, unpack( 'b*', $rout ) );
-        #$eout = 0 unless defined($eout);
-        #my @ebits = split( //, unpack( 'b*', $eout ) );
-        #    print "Ebits: $eout\r\n";
-        foreach my $read_handle (@handles) {
-            if ( $bits[ $read_handle->fileno() ] ) {
+		# Which handles have stuff?
+		my @bits = split( //, unpack( 'b*', $rout ) );
+		$eout = 0 unless defined($eout);
+		my @ebits = split( //, unpack( 'b*', $eout ) );
+
+		#    print "Ebits: $eout\r\n";
+		foreach my $read_handle (@handles) {
+			if ( $bits[ $read_handle->fileno() ] ) {
                 # it would be nice if we could say read until TELNET_GA or the
                 # equivilant, but that's not something we can be sure would be
                 # there.  Cisco doesn't always fill the buffers just because
@@ -853,183 +828,137 @@ sub interconnect {
                 # With colorizing it causes problems because it's data already
                 # written to the screen so you can't take it back (without big
                 # work)
-                $nread = sysread(
-                    $read_handle, ${*$read_handle}{exp_Pty_Buffer},
-                    10240
-                );
+				$nread = sysread(
+					$read_handle, ${*$read_handle}{exp_Pty_Buffer},
+					10240
+				);
 
                 # don't bother trying to colorize input from the user
-                if ($read_handle->fileno() != $self->{stdin_fileno}) {
+                if (${*$read_handle}{exp_Pty_Handle} ne 'STDIN') {
                     foreach my $color (@{$self->{colors}}) {
                         ${*$read_handle}{exp_Pty_Buffer} = $color->colorize(${*$read_handle}{exp_Pty_Buffer});
                     }
                 }
-                # Appease perl -w
-                $nread = 0 unless defined($nread);
-                print STDERR "interconnect: read $nread byte(s) from ${*$read_handle}{exp_Pty_Handle}.\r\n"
-                    if ${*$read_handle}{"exp_Debug"} > 1;
+				# Appease perl -w
+				$nread = 0 unless defined($nread);
+				print STDERR "interconnect: read $nread byte(s) from ${*$read_handle}{exp_Pty_Handle}.\r\n"
+					if ${*$read_handle}{"exp_Debug"} > 1;
 
-                # Test for escape seq. before printing.
-                # Appease perl -w
-                $escape_character_buffer = ''
-                    unless defined($escape_character_buffer);
-                $escape_character_buffer .= ${*$read_handle}{exp_Pty_Buffer};
-                foreach my $escape_sequence ( keys( %{ ${*$read_handle}{exp_Function} } ) ) {
-                    print STDERR "Tested escape sequence $escape_sequence from ${*$read_handle}{exp_Pty_Handle}"
-                        if ${*$read_handle}{"exp_Debug"} > 1;
+				# Test for escape seq. before printing.
+				# Appease perl -w
+				$escape_character_buffer = ''
+					unless defined($escape_character_buffer);
+				$escape_character_buffer .= ${*$read_handle}{exp_Pty_Buffer};
+				foreach my $escape_sequence ( keys( %{ ${*$read_handle}{exp_Function} } ) ) {
+					print STDERR "Tested escape sequence $escape_sequence from ${*$read_handle}{exp_Pty_Handle}"
+						if ${*$read_handle}{"exp_Debug"} > 1;
 
-                    # Make sure it doesn't grow out of bounds.
-                    $escape_character_buffer = $read_handle->_trim_length(
-                        $escape_character_buffer,
-                        ${*$read_handle}{"exp_Max_Accum"}
-                    ) if ( ${*$read_handle}{"exp_Max_Accum"} );
-                    if ( $escape_character_buffer =~ /($escape_sequence)/ ) {
-                        my $match = $1;
-                        if ( ${*$read_handle}{"exp_Debug"} ) {
-                            print STDERR
-                                "\r\ninterconnect got escape sequence from ${*$read_handle}{exp_Pty_Handle}.\r\n";
+					# Make sure it doesn't grow out of bounds.
+					$escape_character_buffer = $read_handle->_trim_length(
+						$escape_character_buffer,
+						${*$read_handle}{"exp_Max_Accum"}
+					) if ( ${*$read_handle}{"exp_Max_Accum"} );
+					if ( $escape_character_buffer =~ /($escape_sequence)/ ) {
+						my $match = $1;
+						if ( ${*$read_handle}{"exp_Debug"} ) {
+							print STDERR
+								"\r\ninterconnect got escape sequence from ${*$read_handle}{exp_Pty_Handle}.\r\n";
 
-                            # I'm going to make the esc. seq. pretty because it will
-                            # probably contain unprintable characters.
-                            print STDERR "\tEscape Sequence: '"
-                                . _trim_length(
-                                undef,
-                                _make_readable($escape_sequence)
-                                ) . "'\r\n";
-                            print STDERR "\tMatched by string: '" . _trim_length( undef, _make_readable($match) ) . "'\r\n";
-                        }
+							# I'm going to make the esc. seq. pretty because it will
+							# probably contain unprintable characters.
+							print STDERR "\tEscape Sequence: '"
+								. _trim_length(
+								undef,
+								_make_readable($escape_sequence)
+								) . "'\r\n";
+							print STDERR "\tMatched by string: '" . _trim_length( undef, _make_readable($match) ) . "'\r\n";
+						}
 
-                        # Print out stuff before the escape.
-                        # Keep in mind that the sequence may have been split up
-                        # over several reads.
-                        # Let's get rid of it from this read. If part of it was
-                        # in the last read there's not a lot we can do about it now.
-                        if ( ${*$read_handle}{exp_Pty_Buffer} =~ /([\w\W]*)($escape_sequence)/ ) {
-                            $read_handle->_print_handles($1);
-                        } else {
-                            $read_handle->_print_handles( ${*$read_handle}{exp_Pty_Buffer} );
-                        }
+						# Print out stuff before the escape.
+						# Keep in mind that the sequence may have been split up
+						# over several reads.
+						# Let's get rid of it from this read. If part of it was
+						# in the last read there's not a lot we can do about it now.
+						if ( ${*$read_handle}{exp_Pty_Buffer} =~ /([\w\W]*)($escape_sequence)/ ) {
+							$read_handle->_print_handles($1);
+						} else {
+							$read_handle->_print_handles( ${*$read_handle}{exp_Pty_Buffer} );
+						}
 
-                        # Clear the buffer so no more matches can be made and it will
-                        # only be printed one time.
-                        ${*$read_handle}{exp_Pty_Buffer} = '';
-                        $escape_character_buffer = '';
+						# Clear the buffer so no more matches can be made and it will
+						# only be printed one time.
+						${*$read_handle}{exp_Pty_Buffer} = '';
+						$escape_character_buffer = '';
 
-                        # Do the function here. Must return non-zero to continue.
-                        # More cool syntax. Maybe I should turn these in to objects.
-                        last CONNECT_LOOP
-                            unless &{ ${ ${*$read_handle}{exp_Function} }{$escape_sequence} }
-                            ( @{ ${ ${*$read_handle}{exp_Parameters} }{$escape_sequence} } );
-                    }
-                }
-                $nread = 0 unless defined($nread); # Appease perl -w?
-                waitpid( ${*$read_handle}{exp_Pid}, WNOHANG )
-                    if ( defined( ${*$read_handle}{exp_Pid} )
-                    && ${*$read_handle}{exp_Pid} );
-                if ( $nread == 0 ) {
-                    print STDERR "Got EOF reading ${*$read_handle}{exp_Pty_Handle}\r\n"
-                        if ${*$read_handle}{"exp_Debug"};
-                    last CONNECT_LOOP
-                        unless defined( ${ ${*$read_handle}{exp_Function} }{"EOF"} );
-                    last CONNECT_LOOP
-                        unless &{ ${ ${*$read_handle}{exp_Function} }{"EOF"} }
-                        ( @{ ${ ${*$read_handle}{exp_Parameters} }{"EOF"} } );
-                }
-                last CONNECT_LOOP if ( $nread < 0 ); # This would be an error
-                $read_handle->_print_handles( ${*$read_handle}{exp_Pty_Buffer} );
-            }
-        }
-    }
-    foreach my $handle (@handles) {
-        unless ( ${*$handle}{"exp_Manual_Stty"} ) {
-            $handle->exp_stty( ${*$handle}{exp_Stored_Stty} );
-        }
-        foreach my $write_handle ( @{ ${*$handle}{exp_Listen_Group} } ) {
-            unless ( ${*$write_handle}{"exp_Manual_Stty"} ) {
-                $write_handle->exp_stty( ${*$write_handle}{exp_Stored_Stty} );
-            }
-        }
-    }
+						# Do the function here. Must return non-zero to continue.
+						# More cool syntax. Maybe I should turn these in to objects.
+						last CONNECT_LOOP
+							unless &{ ${ ${*$read_handle}{exp_Function} }{$escape_sequence} }
+							( @{ ${ ${*$read_handle}{exp_Parameters} }{$escape_sequence} } );
+					}
+				}
+				$nread = 0 unless defined($nread); # Appease perl -w?
+				waitpid( ${*$read_handle}{exp_Pid}, WNOHANG )
+					if ( defined( ${*$read_handle}{exp_Pid} )
+					&& ${*$read_handle}{exp_Pid} );
+				if ( $nread == 0 ) {
+					print STDERR "Got EOF reading ${*$read_handle}{exp_Pty_Handle}\r\n"
+						if ${*$read_handle}{"exp_Debug"};
+					last CONNECT_LOOP
+						unless defined( ${ ${*$read_handle}{exp_Function} }{"EOF"} );
+					last CONNECT_LOOP
+						unless &{ ${ ${*$read_handle}{exp_Function} }{"EOF"} }
+						( @{ ${ ${*$read_handle}{exp_Parameters} }{"EOF"} } );
+				}
+				last CONNECT_LOOP if ( $nread < 0 ); # This would be an error
+				$read_handle->_print_handles( ${*$read_handle}{exp_Pty_Buffer} );
+			}
+		}
+	}
+	foreach my $handle (@handles) {
+		unless ( ${*$handle}{"exp_Manual_Stty"} ) {
+			$handle->exp_stty( ${*$handle}{exp_Stored_Stty} );
+		}
+		foreach my $write_handle ( @{ ${*$handle}{exp_Listen_Group} } ) {
+			unless ( ${*$write_handle}{"exp_Manual_Stty"} ) {
+				$write_handle->exp_stty( ${*$write_handle}{exp_Stored_Stty} );
+			}
+		}
+	}
 
-    return;
+	return;
 }
 
-=head2 run_commands
-
-    $self->run_commands(@commands);
-
-TODO: Document this
-
-
-=cut
-
-sub run_commands {
-    my $self = shift;
-    my $opts = $self->{opts};
-
-    foreach my $arg (@_) {
-        $arg =~ s/\\r/\r/g; # fix for reload\ry.  I believe 'perldoc quotemeta' explains why this happens
-        chomp($arg);
-        $self->send("$arg\r");
-        $self->expect($self->{timeout},'-re', $self->profile->{prompt});
-        sleep($opts->{s}) if ($opts->{s});
-    }
-}
-
-=head2 control_loop
-
-    $self->control_loop();
-
-This is where control should be passed once the session is logged in.  This
-handles CLI commands passed via the -c option, or scripts executed with the -x
-option.  It also handles autocommands passed via either option -a on the
-command line, or via autocmds in the profile.
-
-Calling this without any commands will just run interact()
-
-=cut
-
-sub control_loop {
+sub conn {
     my $self = shift;
+    my $autocmds = shift;
     my $profile = $self->profile;
     my $opts = $self->{opts};
     my $prompt = $profile->{prompt};
-    my $pagercmd = $profile->{pagercmd};
-    my $autocmds;
-    my @args;
 
-    if ($opts->{a}) {
-        $autocmds = [ split(/;/, $opts->{a}) ];
-    } else {
-        $autocmds = $profile->{autocmds};
-    }
-
-    $self->_winch();
-
-    # should -c override -x or be additive? or error if both are specified?
-
-    @args = split(/;/, $opts->{c}) if ($opts->{c});
+    $self->winch();
 
+    my @args = split(/;/, $opts->{c}) if ($opts->{c});
     if ($opts->{x}) {
-        open(my $X, '<', $opts->{x}) || die "Can't open file $opts->{x}\n";
-        @args = <$X>;
-        close $X;
+        open(X, '<', $opts->{x});
+        @args = <X>;
+        close X;
     }
 
     if (@args) {
-        $self->expect($self->{timeout},'-re',$prompt);
-        if (ref($pagercmd) eq 'CODE') {
-            $pagercmd->();
-        } elsif ($pagercmd) {
-            $self->run_commands("$pagercmd\r");
+        $self->expect(10,'-re',$prompt);
+        # this is cisco specific and needs to be abstracted
+        $self->send("term len 0\r");
+        $self->expect(10,'-re',$prompt);
+        foreach my $arg (@args) {
+            $self->send("$arg\r");
+            $self->expect(10,'-re',$prompt);
         }
-        $self->run_commands(@args);
         $self->send($profile->{logoutcmd} ."\r");
     } else {
-        die 'STDIN Not a tty' if (!POSIX::isatty($self->{stdin}));
-        if ($autocmds) {
-            $self->expect($self->{timeout},'-re',$prompt);
-            $self->run_commands(@$autocmds);
+        foreach my $arg (@{$autocmds}) {
+            $self->send("$arg\r");
+            $self->expect(10,'-re',$prompt);
         }
         $self->interact($self->{stdin}, '\cD');
         # q\b is to end anything that's at a More prompt or other dialog and
@@ -1042,26 +971,4 @@ sub control_loop {
     }
 }
 
-=head2 handle_backspace
-
-Handle backspace for routers that use ^H
-
-=cut
-
-sub handle_backspace {
-    ${$_[0]}->send("\b");
-    return 1;
-}
-
-=head2 handle_ctrl_z
-
-Handle ctrl_z for non-cisco boxes
-
-=cut
-
-sub handle_ctrl_z {
-    ${$_[0]}->send("exit\r");
-    return 1;
-}
-
 1;
